package tycho.surefire.problem.chart;

import java.util.List;

import org.eclipse.birt.chart.model.attribute.ActionType;
import org.eclipse.birt.chart.model.attribute.ColorDefinition;
import org.eclipse.birt.chart.model.attribute.Fill;
import org.eclipse.birt.chart.model.attribute.Position;
import org.eclipse.birt.chart.model.attribute.TriggerCondition;
import org.eclipse.birt.chart.model.attribute.impl.TooltipValueImpl;
import org.eclipse.birt.chart.model.component.Series;
import org.eclipse.birt.chart.model.component.impl.SeriesImpl;
import org.eclipse.birt.chart.model.data.SeriesDefinition;
import org.eclipse.birt.chart.model.data.impl.ActionImpl;
import org.eclipse.birt.chart.model.data.impl.NumberDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.SeriesDefinitionImpl;
import org.eclipse.birt.chart.model.data.impl.TextDataSetImpl;
import org.eclipse.birt.chart.model.data.impl.TriggerImpl;
import org.eclipse.birt.chart.model.impl.ChartWithoutAxesImpl;
import org.eclipse.birt.chart.model.type.PieSeries;
import org.eclipse.birt.chart.model.type.impl.PieSeriesImpl;
import org.eclipse.core.runtime.Assert;

public class PieChart
		extends ChartWithoutAxesImpl {
	
	public static final class PieData {
		
		private final Double			value;
		private final String			label;
		private final ColorDefinition	color;
		
		public PieData(final String label, final Double value, final ColorDefinition color) {
			Assert.isTrue(label != null && !label.trim().isEmpty());
			Assert.isNotNull(value);
			Assert.isNotNull(color);
			this.label = label;
			this.value = value;
			this.color = color;
		}
		
	}
	
	public PieChart(final String title, final List<PieData> datas) {
		
		super();
		
		Assert.isTrue(datas != null && !datas.isEmpty());
		
		this.initDefault();
		
		// data
		final String[] labels = new String[datas.size()];
		final Double[] values = new Double[datas.size()];
		final Fill[] fills = new Fill[datas.size()];
		int i = 0;
		for (final PieData data : datas) {
			labels[i] = data.label;
			values[i] = data.value;
			fills[i] = data.color;
			i++;
		}
		
		// Legend
		this.getLegend().setVisible(false);
		
		// Title
		this.getTitle().getLabel().getCaption().setValue(title);
		this.getTitle().getLabel().getCaption().getFont().setSize(10);
		this.getTitle().setVisible(title != null && !title.trim().isEmpty());
		
		// Base Series
		final SeriesDefinition baseSeriesDefintion = SeriesDefinitionImpl.create();
		
		final Series categorySeries = SeriesImpl.create();
		categorySeries.setDataSet(TextDataSetImpl.create(labels));
		baseSeriesDefintion.getSeries().add(categorySeries);
		
		baseSeriesDefintion.getSeriesPalette().getEntries().clear();
		for (final Fill element : fills) {
			baseSeriesDefintion.getSeriesPalette().getEntries().add(element);
		}
		
		// Orthogonal Series
		final SeriesDefinition seriesDefinition = SeriesDefinitionImpl.create();
		
		final PieSeries series = (PieSeries) PieSeriesImpl.create();
		series.setDataSet(NumberDataSetImpl.create(values));
		series.setLabelPosition(Position.INSIDE_LITERAL);
		series.getLabel().getCaption().getFont().setSize(10);
		series.getLabel().getCaption().getFont().setBold(true);
		series.getTitle().setVisible(false);
		series.setInnerRadius(40);
		series.setExplosion(1);
		
		series.getTriggers().add(TriggerImpl.create(TriggerCondition.ONMOUSEOVER_LITERAL, ActionImpl.create(ActionType.SHOW_TOOLTIP_LITERAL, TooltipValueImpl.create(500, null))));
		
		seriesDefinition.getSeries().add(series);
		
		baseSeriesDefintion.getSeriesDefinitions().add(seriesDefinition);
		
		this.getSeriesDefinitions().add(baseSeriesDefintion);
		
	}
}
